package engine.QuestSystem;

import engine.InterestManagement.WorldGrid;
import engine.gameManager.ChatManager;
import engine.net.client.msg.ErrorPopupMsg;
import engine.objects.*;
import engine.server.MBServerStatics;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Random;

public class QuestManager {
    public static HashMap<PlayerCharacter,QuestObject> acceptedQuests = new HashMap<>();
    public static HashMap<PlayerCharacter,ArrayList<String>> completedQuests = new HashMap<>();

    public static boolean grantsQuest(NPC npc){
        if(npc == null)
            return false;

        if(npc.contract == null)
            return false;

        return npc.contract.getName().contains("ZoneLore") || npc.contract.getName().equals("Barrowlands Sentry");
    }

    public static void displayCurrentQuest(PlayerCharacter pc){
        if(acceptedQuests.containsKey(pc)){
            QuestObject obj = acceptedQuests.get(pc);
            String output = "You have embarked on the noble quest: ";
            output += obj.QuestName + ". ";
            output += obj.description;
            output += ". " + obj.objectiveCount + " / " + obj.objectiveCountRequired;
            ErrorPopupMsg.sendErrorMsg(pc, output);
        }else{
            ErrorPopupMsg.sendErrorMsg(pc, "You have no active quest.");
        }
    }

    public static void acceptQuest(PlayerCharacter pc, QuestObject obj){
        if (completedQuests.containsKey(pc)) {
            if(completedQuests.get(pc).contains(obj.QuestName)){
                ErrorPopupMsg.sendErrorMsg(pc, "You have already completed the quest: " + obj.QuestName);
                return;
            }
        }

        if(acceptedQuests.containsKey(pc)){
            if(acceptedQuests.get(pc)!= null){
                ErrorPopupMsg.sendErrorMsg(pc, "You have already embarked on a noble quest.");
                return;
            }
        }

        acceptedQuests.put(pc,obj);
        displayCurrentQuest(pc);
    }

    public static void completeQuest(PlayerCharacter pc, QuestObject obj){

        if(obj.objectiveCount < obj.objectiveCountRequired) {
            ChatManager.chatSystemInfo(pc, obj.QuestName + " Progress: " + obj.objectiveCount + " / " + obj.objectiveCountRequired);
            return;
        }

        //notify the player they have completed their quest
        ErrorPopupMsg.sendErrorMsg(pc, "You have completed the quest: " + obj.QuestName +"!");

        //add completed quest to completion log
        if (completedQuests.containsKey(pc)) {
            completedQuests.get(pc).add(obj.QuestName);
        }else{
            ArrayList<String> completed = new ArrayList<>();
            completed.add(obj.QuestName);
            completedQuests.put(pc,completed);
        }

        //remove current quest from active log
        if(acceptedQuests.containsKey(pc))
            acceptedQuests.remove(pc);

        //grant rewards
        //only grant XP if level is below 75
        if(pc.level < 75) {
            int xpGrant = (int) (Experience.maxXPPerKill(pc.getLevel()) * 10);
            pc.grantXP(xpGrant);
            ChatManager.chatSystemInfo(pc, "Your Quest Has Granted you: " + xpGrant + " Experience Points");
        }

        int goldGrant = (int) Experience.maxXPPerKill(pc.getLevel());
        pc.getCharItemManager().addGoldToInventory(goldGrant,false);
        ChatManager.chatSystemInfo(pc, "Your Quest Has Granted you: " + goldGrant + " Gold Coins");

        pc.getCharItemManager().updateInventory();
    }

    public static QuestObject getQuestForContract(NPC npc){
        QuestObject obj = new QuestObject();
        obj.QuestName = npc.getFirstName() + "'s Quest";

        HashSet<AbstractWorldObject> mobs = WorldGrid.getObjectsInRangePartial(npc.loc,2048, MBServerStatics.MASK_MOB);
        if (mobs.isEmpty())
            return null; // Handle the case where the set is empty

        // Convert HashSet to a List
        ArrayList<AbstractWorldObject> mobList = new ArrayList<>(mobs);

        Mob mob = null;

        while(mob == null || Mob.discDroppers.contains(mob)) {
            // Generate a random index
            Random random = new Random();
            int randomIndex = random.nextInt(mobList.size());

            if (mobList.get(randomIndex) == null) {
                continue;
            }

            mob = (Mob) mobList.get(randomIndex);
        }
        obj.progressionNames = new ArrayList<>();
        obj.progressionNames.add(mob.getFirstName());

        obj.objectiveCountRequired = 10;
        obj.objectiveCount = 0;

        obj.description = "These lands are plagued with " + mob.getFirstName() + " complete the quest by slaying 10 of them!";
        return obj;
    }
}
